#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Apr  4 10:35:32 2019

@author: martindrech
"""

from PyQt5.QtWidgets import QMainWindow, QPushButton, QLabel

from pyqtgraph.Qt import QtGui, QtCore
from pyqtgraph import dockarea
import pyqtgraph as pg
from zylaCameraWorker import CameraWorker

import sys
import numpy as np


class GuiMainWindow(QMainWindow):


     def __init__(self):
        super().__init__()
        
        
        #self.__init__
        
        self.counter = 0
        self.frames_checker_timer = QtCore.QTimer()
        
        self.initUI()
        
     def initUI(self):  
        
        self.win = QtGui.QMainWindow()
        self.win.setWindowTitle('Toy GUI')
        self.win.resize(1000,500)
    
        self.area = dockarea.DockArea()
        self.win.setCentralWidget(self.area)
        self.cw = QtGui.QWidget()
        self.win.show()
        

           
        self.d1 = dockarea.Dock("Zyla Camera Control")
        self.d2 = dockarea.Dock("Zyla Camera Analysis")
        
        self.area.addDock(self.d1) 
        self.area.addDock(self.d2, 'left', self.d1) 
                
        ## button spinbox and label
        self.cameraButton = QPushButton('Acquisition')
        self.d1.addWidget(self.cameraButton)
        self.sb = pg.SpinBox(min = 0)
        self.d1.addWidget(self.sb)
        self.label = QLabel('nada')
        self.d1.addWidget(self.label)
        
        #layouts and viewboxes
        self.ImageLayout = pg.GraphicsLayoutWidget()
        self.view = self.ImageLayout.addViewBox()
        self.AnalysisLayout = pg.GraphicsLayoutWidget()
        
        #image layout
        self.img = pg.ImageItem(border='w')
        self.imv = pg.ImageView(imageItem = self.img, view = self.view)
        self.d1.addWidget(self.imv)
    
        #analysis layout
        self.d2.addWidget(self.AnalysisLayout)
        self.p = self.AnalysisLayout.addPlot(row = 0, col = 0,  title="Time plot")
        self.p.setRange(QtCore.QRectF(0, 0, 50, 255)) 
        self.p.setAutoPan(y=True)
        #self.p.setRange(xRange = (0, 10), yRange = (0, 255))
        self.curve = self.p.plot(pen='y')
               
               
        ## rOI
        self.rois = []
        self.rois.append(pg.TestROI([0,  0], [10, 10], pen=(0,9)))
        for r in self.rois:
            self.view.addItem(r)
            #r.sigRegionChanged.connect(self.updateRoi)
            
        self.ROIdata = np.zeros(100)
        self.iROIdata = 0
        
        self._initialize_image()
       
        
        
     def make_connections(self, backend):
         ##backend connections
         backend.imageReadySignal.connect(self.updateImage)
         
         ##internal connections
         self.frames_checker_timer.timeout.connect(self.frame_counter)
         self.cameraButton.clicked.connect(lambda x = backend.isCameraAcquiring: self.camera_button_pressed(x))
         
     def camera_button_pressed(self, isCameraAcquiring):
         
         if isCameraAcquiring:
             self.frames_checker_timer.stop()
         else:
             self.frame_index = 0
             self.previous_frame_index = 0
             self.frames_checker_timer.start(1000)
             
     def _initialize_image(self):
        ## lock the aspect ratio so pixels are always square
        self.view.setAspectLocked(True)        
        self.view.addItem(self.img)
        
        self.frame_index = 0
        self.previous_frame_index = 0
        self.frames_checker_timer.start(1000)
    
     def updateRois(self):
        
        for roi in self.rois:
        
            self.lastRoi = roi
            roiSlice = roi.getArrayRegion(self.img.image, img=self.img)
            self.newData = np.mean(roiSlice)
            self.ROIdata[self.iROIdata] = self.newData
            self.iROIdata = np.mod(self.frame_index + 1, len(self.ROIdata))
            self.curve.setData(self.ROIdata)
            
        
     @QtCore.pyqtSlot(np.ndarray, int)     
     def updateImage(self, image, acq_index):
        
        if self.frame_index is not 0 and not self.frame_index == acq_index:
            print('Dephasing in camera gui has occured: while frame index is %i, the acq index is %i' % (self.frame_index, acq_index))
        
        #n = 1024
        self.img.setImage(image)
        self.updateRois()
        self.frame_index = self.frame_index + 1
        
        
    
     def frame_counter(self):
         self.counter = self.counter + 1
         self.label.setText('Frames in last second: ' + str(self.frame_index-self.previous_frame_index) + '    ' + 'Seconds since start: ' + str(self.counter))
         self.previous_frame_index = self.frame_index
         
         
     def __del__(self):
         print('chau gui')
         
###############################################################################

if __name__ == '__main__':
    app = QtGui.QApplication([])
    
    myGuiMainWIndow = GuiMainWindow()
    cameraWorker = CameraWorker()
    
    cameraWorker.make_connections(myGuiMainWIndow)
    myGuiMainWIndow.make_connections(cameraWorker)
    
    cameraThread = QtCore.QThread() 
    cameraWorker.moveToThread(cameraThread)
    
    cameraThread.start()

    
    
    sys.exit(app.exec_())